@@ -24,24 +24,26 @@ local logger = require "rspamd_logger"
 
 -- Check rule for a single email
 local function check_email_rule(task, rule, addr)
-  local function emails_dns_cb(resolver, to_resolve, results, err)
-    if results then
+  local function emails_dns_cb(_, to_resolve, results, err)
+    if err and (err ~= 'requested record is not found' and err ~= 'no records with this name') then
+      logger.errx(task, 'Error querying DNS: %1', err)
+    elseif results then
       logger.infox(task, '<%1> email: [%2] resolved for symbol: %3',
         task:get_message_id(), to_resolve, rule['symbol'])
       task:insert_result(rule['symbol'], 1)
     end
   end
   if rule['dnsbl'] then
-    local to_resolve = ''
+    local to_resolve
     if rule['domain_only'] then
       to_resolve = string.format('%s.%s', addr:get_host(), rule['dnsbl'])
     else
       to_resolve = string.format('%s.%s.%s', addr:get_user(), addr:get_host(), rule['dnsbl'])
     end
-    
+
     task:get_resolver():resolve_a({
-      task=task, 
-      name = to_resolve, 
+      task=task,
+      name = to_resolve,
       callback = emails_dns_cb})
   elseif rule['map'] then
     if rule['domain_only'] then
@@ -74,22 +76,29 @@ local function check_emails(task)
           check_email_rule(task, rule, addr)
         end
         checked[to_check] = true
-      end 
+      end
     end
   end
 end
 
 local opts =  rspamd_config:get_all_opt('emails', 'rule')
 if opts and type(opts) == 'table' then
-  for k,v in pairs(opts) do
-    if k == 'rule' and type(v) == 'table' then
+  local r = opts['rule']
+
+  if r then
+    for k,v in pairs(r) do
       local rule = v
       if not rule['symbol'] then
         rule['symbol'] = k
       end
+
       if rule['map'] then
         rule['name'] = rule['map']
-        rule['map'] = rspamd_config:add_hash_map (rule['name'])
+        rule['map'] = rspamd_config:add_map({
+           url = rule['name'],
+           description = string.format('Emails rule %s', rule['symbol']),
+           type = 'regexp'
+        })
       end
       if not rule['symbol'] or (not rule['map'] and not rule['dnsbl']) then
         logger.errx(rspamd_config, 'incomplete rule')
@@ -100,10 +109,17 @@ if opts and type(opts) == 'table' then
   end
 end
 
-if table.maxn(rules) > 0 then
+if #rules > 0 then
   -- add fake symbol to check all maps inside a single callback
-  local id = rspamd_config:register_callback_symbol(1.0, check_emails)
+  local id = rspamd_config:register_symbol({
+    type = 'callback',
+    callback = check_emails
+  })
   for _,rule in ipairs(rules) do
-    rspamd_config:register_virtual_symbol(rule['symbol'], 1.0, id)
+    rspamd_config:register_symbol({
+      name = rule['symbol'],
+      type = 'virtual',
+      parent = id
+    })
   end
 end
