@@ -1,5 +1,5 @@
 --[[
-Copyright (c) 2011-2015, Vsevolod Stakhov <vsevolod@highsecure.ru>
+Copyright (c) 2011-2016, Vsevolod Stakhov <vsevolod@highsecure.ru>
 Copyright (c) 2013-2015, Alexey Savelyev <info@homeweb.ru>
 
 Licensed under the Apache License, Version 2.0 (the "License");
@@ -19,10 +19,8 @@ limitations under the License.
 -- Weight for checks_hellohost and checks_hello: 5 - very hard, 4 - hard, 3 - meduim, 2 - low, 1 - very low.
 -- From HFILTER_HELO_* and HFILTER_HOSTNAME_* symbols the maximum weight is selected in case of their actuating.
 
-
---local dumper = require 'pl.pretty'.dump
-local rspamd_regexp = require "rspamd_regexp"
 local rspamd_logger = require "rspamd_logger"
+local rspamd_regexp = require "rspamd_regexp"
 local rspamc_local_helo = "rspamc.local"
 local checks_hellohost = {
   ['[.-]gprs[.-]'] = 5, ['gprs[.-][0-9]'] = 5, ['[0-9][.-]?gprs'] = 5,
@@ -65,34 +63,34 @@ local checks_hellohost = {
 local checks_hello = {
   ['^[^\\.]+$'] = 5, -- for helo=COMPUTER, ANNA, etc... Without dot in helo
   ['^(dsl)?(device|speedtouch)\\.lan$'] = 5,
-  ['\\.(lan|local|home|localdomain|intra|in-addr.arpa|priv|online|user|veloxzon)$'] = 5
+  ['\\.(lan|local|home|localdomain|intra|in-addr.arpa|priv|user|veloxzon)$'] = 5
 }
 
 local checks_hello_badip = {
-  ['^0\\.'] = 5,
-  ['^::1$'] = 5, --loopback ipv4, ipv6
-  ['^127\\.'] = 5,
-  ['^10\\.'] = 5,
-  ['^192\\.168\\.'] = 5, --local ipv4
-  ['^172\\.1[6-9]\\.'] = 5,
-  ['^172\\.2[0-9]\\.'] = 5,
-  ['^172\\.3[01]\\.'] = 5,  --local ipv4
-  ['^169\\.254\\.'] = 5, --chanel ipv4
-  ['^192\\.0\\.0\\.'] = 5, --IETF Protocol
-  ['^192\\.88\\.99\\.'] = 5, --RFC3068
-  ['^100.6[4-9]\\.'] = 5,
-  ['^100.[7-9]\\d\\.'] = 5,
-  ['^100.1[01]\\d\\.'] = 5,
-  ['^100.12[0-7]\\d\\.'] = 5, --RFC6598
-  ['^\\d\\.\\d\\.\\d\\.255$'] = 5, --multicast ipv4
-  ['^192\\.0\\.2\\.'] = 5,
-  ['^198\\.51\\.100\\.'] = 5,
-  ['^203\\.0\\.113\\.'] = 5,  --sample
-  ['^fe[89ab][0-9a-f]::'] = 5,
-  ['^fe[cdf][0-9a-f]:'] = 5, --local ipv6 (fe80:: - febf::, fec0:: - feff::)
-  ['^2001:db8::'] = 5, --reserved RFC 3849 for ipv6
-  ['^fc00::'] = 5,
-  ['^ffxx::'] = 5 --unicast, multicast ipv6
+  ['^0\\.'] = 1,
+  ['^::1$'] = 1, --loopback ipv4, ipv6
+  ['^127\\.'] = 1,
+  ['^10\\.'] = 1,
+  ['^192\\.168\\.'] = 1, --local ipv4
+  ['^172\\.1[6-9]\\.'] = 1,
+  ['^172\\.2[0-9]\\.'] = 1,
+  ['^172\\.3[01]\\.'] = 1,  --local ipv4
+  ['^169\\.254\\.'] = 1, --chanel ipv4
+  ['^192\\.0\\.0\\.'] = 1, --IETF Protocol
+  ['^192\\.88\\.99\\.'] = 1, --RFC3068
+  ['^100.6[4-9]\\.'] = 1,
+  ['^100.[7-9]\\d\\.'] = 1,
+  ['^100.1[01]\\d\\.'] = 1,
+  ['^100.12[0-7]\\d\\.'] = 1, --RFC6598
+  ['^\\d\\.\\d\\.\\d\\.255$'] = 1, --multicast ipv4
+  ['^192\\.0\\.2\\.'] = 1,
+  ['^198\\.51\\.100\\.'] = 1,
+  ['^203\\.0\\.113\\.'] = 1,  --sample
+  ['^fe[89ab][0-9a-f]::'] = 1,
+  ['^fe[cdf][0-9a-f]:'] = 1, --local ipv6 (fe80:: - febf::, fec0:: - feff::)
+  ['^2001:db8::'] = 1, --reserved RFC 3849 for ipv6
+  ['^fc00::'] = 1,
+  ['^ffxx::'] = 1 --unicast, multicast ipv6
 }
 
 local checks_hello_bareip = {
@@ -113,6 +111,9 @@ local config = {
   ['url_enabled'] = false
 }
 
+local check_local = false
+local check_authed = false
+
 local function check_regexp(str, regexp_text)
   if not compiled_regexp[regexp_text] then
     compiled_regexp[regexp_text] = rspamd_regexp.create(regexp_text, 'i')
@@ -125,31 +126,6 @@ local function check_regexp(str, regexp_text)
   return false
 end
 
-local function split(str, delim, maxNb)
-  -- Eliminate bad cases...
-  if string.find(str, delim) == nil then
-    return { str }
-  end
-  if maxNb == nil or maxNb < 1 then
-    maxNb = 0    -- No limit
-  end
-  local result = {}
-  local pat = "(.-)" .. delim .. "()"
-  local nb = 0
-  local lastPos
-  for part, pos in string.gmatch(str, pat) do
-    nb = nb + 1
-    result[nb] = part
-    lastPos = pos
-    if nb == maxNb then break end
-  end
-  -- Handle the last field
-  if nb ~= maxNb then
-    result[nb + 1] = string.sub(str, lastPos)
-  end
-  return result
-end
-
 local function check_fqdn(domain)
   if check_regexp(domain, '(?=^.{4,253}$)(^((?!-)[a-zA-Z0-9-]{1,63}(?<!-)\\.)+[a-zA-Z0-9-]{2,63}\\.?$)') then
     return true
@@ -165,8 +141,11 @@ local function check_host(task, host, symbol_suffix, eq_ip, eq_host)
   local failed_address = 0
   local resolved_address = {}
 
-  local function check_host_cb_mx(resolver, to_resolve, results, err)
+  local function check_host_cb_mx(_, to_resolve, results, err)
     task:inc_dns_req()
+    if err and (err ~= 'requested record is not found' and err ~= 'no records with this name') then
+        rspamd_logger.errx(task, 'error looking up %s: %s', to_resolve, err)
+    end
     if not results then
       task:insert_result('HFILTER_' .. symbol_suffix .. '_NORES_A_OR_MX', 1.0)
     else
@@ -174,10 +153,10 @@ local function check_host(task, host, symbol_suffix, eq_ip, eq_host)
         if mx['name'] then
           local failed_mx_address = 0
           -- Capture failed_mx_address
-          local function check_host_cb_mx_a(resolver, to_resolve, results, err)
+          local function check_host_cb_mx_a(_, _, mx_results)
             task:inc_dns_req()
 
-            if not results then
+            if not mx_results then
               failed_mx_address = failed_mx_address + 1
             end
 
@@ -200,7 +179,7 @@ local function check_host(task, host, symbol_suffix, eq_ip, eq_host)
       end
     end
   end
-  local function check_host_cb_a(resolver, to_resolve, results, err)
+  local function check_host_cb_a(_, _, results)
     task:inc_dns_req()
 
     if not results then
@@ -267,34 +246,51 @@ local function hfilter(task)
   if config['url_enabled'] then
     local parts = task:get_text_parts()
     if parts then
-      --One text part--
-      local total_parts_len = 0
-      local text_parts_count = 0
-      local lines_max = 0
-      local selected_text_part = nil
+      local plain_text_part = nil
+      local html_text_part = nil
       for _,p in ipairs(parts) do
-        total_parts_len = total_parts_len + p:get_length()
-
-        if p:get_lines_count() > lines_max then
-          lines_max = p:get_lines_count()
-        end
-
-        if not p:is_html() then
-          text_parts_count = text_parts_count + 1
-          selected_text_part = p
+        if p:is_html() then
+          html_text_part = p
+        else
+          plain_text_part = p
         end
       end
-      if total_parts_len > 0 then
-        local urls = task:get_urls()
-        if urls then
-          local total_url_len = 0
-          for _,url in ipairs(urls) do
-            total_url_len = total_url_len + url:get_length()
+      local hc = nil
+      if html_text_part then
+        hc = html_text_part:get_html()
+        if hc then
+          local url_len = 0
+          hc:foreach_tag('a', function(_, len)
+            url_len = url_len + len
+            return false
+          end)
+
+          local plen = html_text_part:get_length()
+
+          if url_len > 0 and plen > 0 then
+            local rel = url_len / plen
+            if rel > 0.8 then
+              local sc = (rel - 0.8) * 5.0
+              if sc > 1.0 then sc = 1.0 end
+              task:insert_result('HFILTER_URL_ONLY', sc)
+              local lines =  html_text_part:get_lines_count()
+              if lines > 0 and lines < 2 then
+                task:insert_result('HFILTER_URL_ONELINE', 1.00)
+              end
+            end
           end
-          if total_url_len > 0 then
-            if total_url_len + 7 > total_parts_len then
-              task:insert_result('HFILTER_URL_ONLY', 1.00)
-            elseif lines_max > 0 and lines_max < 2 then
+        end
+      end
+      if not hc and plain_text_part then
+        local url_len = plain_text_part:get_urls_length()
+        local plen = plain_text_part:get_length()
+
+        if plen > 0 and url_len > 0 then
+          local rel = url_len / plen
+          if rel > 0.8 then
+            task:insert_result('HFILTER_URL_ONLY', (rel - 0.8) * 5.0)
+            local lines = plain_text_part:get_lines_count()
+            if lines > 0 and lines < 2 then
               task:insert_result('HFILTER_URL_ONELINE', 1.00)
             end
           end
@@ -303,22 +299,24 @@ local function hfilter(task)
     end
   end
 
-  --No more checks for auth user
-  if task:get_user() ~= nil then
+  --No more checks for auth user or local network
+  local rip = task:get_from_ip()
+  if ((not check_authed and task:get_user()) or
+      (not check_local and rip and rip:is_local())) then
     return false
   end
 
   --local message = task:get_message()
   local ip = false
-  local rip = task:get_from_ip()
   if rip and rip:is_valid() then
     ip = rip:to_string()
   end
 
   -- Check's HELO
   local weight_helo = 0
+  local helo
   if config['helo_enabled'] then
-    local helo = task:get_helo()
+    helo = task:get_helo()
     if helo then
       if helo ~= rspamc_local_helo then
         helo = string.gsub(helo, '[%[%]]', '')
@@ -326,7 +324,7 @@ local function hfilter(task)
         local find_badip = false
         for regexp,weight in pairs(checks_hello_badip) do
           if check_regexp(helo, regexp) then
-            task:insert_result('HFILTER_HELO_BADIP', 1.0)
+            task:insert_result('HFILTER_HELO_BADIP', weight)
             find_badip = true
             break
           end
@@ -367,8 +365,6 @@ local function hfilter(task)
           end
         end
       end
-    else
-      task:insert_result('HFILTER_HELO_UNKNOWN', 1.0)
     end
   end
 
@@ -388,8 +384,6 @@ local function hfilter(task)
           end
         end
       end
-    else
-      task:insert_result('HFILTER_HOSTNAME_UNKNOWN', 1.00)
     end
   end
 
@@ -407,8 +401,8 @@ local function hfilter(task)
     if from then
       --FROM host check
       for _,fr in ipairs(from) do
-        local fr_split = split(fr['addr'], '@', 0)
-        if table.maxn(fr_split) == 2 then
+        local fr_split = rspamd_str_split(fr['addr'], '@')
+        if #fr_split == 2 then
           check_host(task, fr_split[2], 'FROMHOST', '', '')
           if fr_split[1] == 'postmaster' then
             frombounce = true
@@ -427,7 +421,7 @@ local function hfilter(task)
   if config['rcpt_enabled'] then
     local rcpt = task:get_recipients()
     if rcpt then
-      local count_rcpt = table.maxn(rcpt)
+      local count_rcpt = #rcpt
       if frombounce then
         if count_rcpt > 1 then
           task:insert_result('HFILTER_RCPT_BOUNCEMOREONE', 1.00)
@@ -440,8 +434,8 @@ local function hfilter(task)
   if config['mid_enabled'] then
     local message_id = task:get_message_id()
     if message_id then
-      local mid_split = split(message_id, '@', 0)
-      if table.maxn(mid_split) == 2 and not string.find(mid_split[2], 'local') then
+      local mid_split = rspamd_str_split(message_id, '@')
+      if #mid_split == 2 and not string.find(mid_split[2], 'local') then
         check_host(task, mid_split[2], 'MID')
       end
     end
@@ -455,7 +449,6 @@ local symbols_enabled = {}
 local symbols_helo = {
   "HFILTER_HELO_BAREIP",
   "HFILTER_HELO_BADIP",
-  "HFILTER_HELO_UNKNOWN",
   "HFILTER_HELO_1",
   "HFILTER_HELO_2",
   "HFILTER_HELO_3",
@@ -493,7 +486,17 @@ local symbols_from = {
   "HFILTER_FROM_BOUNCE"
 }
 
-local opts = rspamd_config:get_all_opt('hfilter')
+local opts = rspamd_config:get_all_opt('options')
+if opts and type(opts) ~= 'table' then
+  if type(opts['check_local']) == 'boolean' then
+    check_local = opts['check_local']
+  end
+  if type(opts['check_authed']) == 'boolean' then
+    check_authed = opts['check_authed']
+  end
+end
+
+opts = rspamd_config:get_all_opt('hfilter')
 if opts then
   for k,v in pairs(opts) do
     config[k] = v
@@ -523,6 +526,6 @@ if config['url_enabled'] then
 end
 
 --dumper(symbols_enabled)
-if table.maxn(symbols_enabled) > 0 then
+if #symbols_enabled > 0 then
   rspamd_config:register_symbols(hfilter, 1.0, "HFILTER", symbols_enabled);
 end
