@@ -16,21 +16,17 @@ limitations under the License.
 
 -- IP score is a module that set ip score of specific ip, asn, country
 local rspamd_logger = require "rspamd_logger"
-local rspamd_redis = require "rspamd_redis"
-local upstream_list = require "rspamd_upstream_list"
 local rspamd_regexp = require "rspamd_regexp"
 local rspamd_util = require "rspamd_util"
-local _ = require "fun"
 
 -- Default settings
-local default_port = 6379
-local upstreams = nil
+local redis_params = nil
 local whitelist = nil
 local asn_cc_whitelist = nil
+local check_authed = false
+local check_local = false
 
 local options = {
-  asn_provider = 'origin.asn.cymru.com', -- provider for ASN data
-  asn6_provider = 'origin6.asn.cymru.com', -- provider for ASN data
   actions = { -- how each action is treated in scoring
     ['reject'] = 1.0,
     ['add header'] = 0.25,
@@ -52,43 +48,12 @@ local options = {
   lower_bound = 10, -- minimum number of messages to be scored
   metric = 'default',
   min_score = nil,
-  max_score = nil
+  max_score = nil,
+  score_divisor = 1,
 }
 
 local asn_re = rspamd_regexp.create_cached("[\\|\\s]")
 
-local function asn_check(task)
-  local ip = task:get_from_ip()
-
-  local function asn_dns_cb(resolver, to_resolve, results, err, key)
-    if results and results[1] then
-      local parts = asn_re:split(results[1])
-      -- "15169 | 8.8.8.0/24 | US | arin |" for 8.8.8.8
-      if parts[1] then
-        task:get_mempool():set_variable("asn", parts[1])
-      end
-      if parts[2] then
-        task:get_mempool():set_variable("ipnet", parts[2])
-      end
-      if parts[3] then
-        task:get_mempool():set_variable("country", parts[3])
-      end
-    end
-  end
-
-  if ip and ip:is_valid() then
-    local asn_provider = 'asn_provider'
-    if ip:get_version() == 6 then
-      asn_provider = 'asn6_provider'
-    end
-    local req_name = rspamd_logger.slog("%1.%2",
-      table.concat(ip:inversed_str_octets(), '.'), options[asn_provider])
-
-    task:get_resolver():resolve_txt(task:get_session(), task:get_mempool(),
-        req_name, asn_dns_cb)
-  end
-end
-
 local function ip_score_hash_key(asn, country, ipnet, ip)
   -- We use the most common attribute as hashing key
   if country then
@@ -118,30 +83,47 @@ local function ip_score_get_task_vars(task)
   return asn, country, ipnet
 end
 
+local function normalize_score(sc, total, mult)
+  if total < options['lower_bound'] then
+    return 0
+  end
+  return mult * rspamd_util.tanh(2.718281 * sc / total)
+end
+
 -- Set score based on metric's action
 local ip_score_set = function(task)
   local function new_score_set(score, old_score, old_total)
     local new_total
-    if old_total == -1 then
+    if old_total == -1 or old_total ~= old_total then
       new_total = 1
     else
       new_total = old_total + 1
     end
 
+    if score ~= score then
+      score = 0
+    end
+
     return old_score + score, new_total
   end
 
-  local score_set_cb = function(task, err, data)
+  local score_set_cb = function(err)
     if err then
       rspamd_logger.infox(task, 'got error while IP score changing: %1', err)
     end
   end
 
-  local action = task:get_metric_action(options['metric'])
   local ip = task:get_from_ip()
+  if not check_authed and task:get_user() then
+    return
+  end
+  local action = task:get_metric_action(options['metric'])
   if not ip or not ip:is_valid() then
     return
   end
+  if not check_local and ip:is_local() then
+    return
+  end
 
   local pool = task:get_mempool()
   local asn, country, ipnet = ip_score_get_task_vars(task)
@@ -165,47 +147,94 @@ local ip_score_set = function(task)
     score_mult = 0
   end
 
-  score = score_mult * rspamd_util.tanh (2.718 * score)
+  score = score_mult * rspamd_util.tanh (2.718281 * (score/options['score_divisor']))
 
   local hkey = ip_score_hash_key(asn, country, ipnet, ip)
-  local upstream = upstreams:get_upstream_by_hash(hkey)
-  local addr = upstream:get_addr()
 
   asn_score,total_asn = new_score_set(score, asn_score, total_asn)
   country_score,total_country = new_score_set(score, country_score, total_country)
   ipnet_score,total_ipnet = new_score_set(score, ipnet_score, total_ipnet)
   ip_score,total_ip = new_score_set(score, ip_score, total_ip)
-
-  rspamd_redis.make_request(task, addr, score_set_cb,
-    'HMSET', {options['hash'],
+  local redis_args = {options['hash'],
     options['asn_prefix'] .. asn, string.format('%f|%d', asn_score, total_asn),
     options['country_prefix'] .. country, string.format('%f|%d', country_score, total_country),
     options['ipnet_prefix'] .. ipnet, string.format('%f|%d', ipnet_score, total_ipnet),
-    ip:to_string(), string.format('%f|%d', ip_score, total_ip)})
+    ip:to_string(), string.format('%f|%d', ip_score, total_ip)}
+
+  local ret = rspamd_redis_make_request(task,
+    redis_params, -- connect params
+    hkey, -- hash key
+    true, -- is write
+    score_set_cb, --callback
+    'HMSET', -- command
+    redis_args -- arguments
+  )
+  if not ret then
+    rspamd_logger.errx(task, 'error connecting to redis')
+    return
+  end
+
+  -- Now insert final result
+  asn_score = normalize_score(asn_score, total_asn, options['scores']['asn'])
+  country_score = normalize_score(country_score, total_country,
+      options['scores']['country'])
+  ipnet_score = normalize_score(ipnet_score, total_ipnet,
+       options['scores']['ipnet'])
+  ip_score = normalize_score(ip_score, total_ip, options['scores']['ip'])
+
+  local total_score = 0.0
+  local description_t = {}
+
+  if ip_score ~= 0 then
+    total_score = total_score + ip_score
+    table.insert(description_t, 'ip: ' .. '(' .. string.format('%.2f', ip_score * 10) .. ')')
+  end
+  if ipnet_score ~= 0 then
+    total_score = total_score + ipnet_score
+    table.insert(description_t, 'ipnet: ' .. ipnet .. '(' .. string.format('%.2f', ipnet_score * 10) .. ')')
+  end
+  if asn_score ~= 0 then
+    total_score = total_score + asn_score
+    table.insert(description_t, 'asn: ' .. asn .. '(' .. string.format('%.2f', asn_score * 10) .. ')')
+  end
+  if country_score ~= 0 then
+    total_score = total_score + country_score
+    table.insert(description_t, 'country: ' .. country .. '(' .. string.format('%.2f', country_score * 10) .. ')')
+  end
+
+  if options['max_score'] and (total_score*10) > options['max_score'] then
+    total_score = options['max_score']/10
+  end
+  if options['min_score'] and (total_score*10) < options['min_score'] then
+    total_score = options['min_score']/10
+  end
+
+  if total_score ~= 0 then
+    task:insert_result(options['symbol'], total_score, table.concat(description_t, ', '))
+  end
 end
 
 -- Check score for ip in keystorage
 local ip_score_check = function(task)
   local asn, country, ipnet = ip_score_get_task_vars(task)
+  local ip = task:get_from_ip()
 
-  local ip_score_redis_cb = function(task, err, data)
+  local ip_score_redis_cb = function(err, data)
+    if err then
+      rspamd_logger.errx(task, 'Redis error: %s', err)
+      -- XXX: upstreams
+    end
     local function calculate_score(score)
       local parts = asn_re:split(score)
       local rep = tonumber(parts[1])
       local total = tonumber(parts[2])
 
+      if rep ~= rep then rep = 0 end
+      if total ~= total then total = 0 end
+
       return rep, total
     end
 
-    local function normalize_score(sc, total, mult)
-      if total < options['lower_bound'] then
-        return 0
-      end
-
-      -- -mult to mult
-      return mult * rspamd_util.tanh(2.718 * sc / total)
-    end
-
     if err then
       -- Key is not found or error occurred
       return
@@ -232,44 +261,10 @@ local ip_score_check = function(task)
         country_score,total_country,
         ipnet_score,total_ipnet,
         ip_score, total_ip)
-
-      asn_score = normalize_score(asn_score, total_asn, options['scores']['asn'])
-      country_score = normalize_score(country_score, total_country,
-        options['scores']['country'])
-      ipnet_score = normalize_score(ipnet_score, total_ipnet,
-        options['scores']['ipnet'])
-      ip_score = normalize_score(ip_score, total_ip, options['scores']['ip'])
-
-      local total_score = 0.0
-      local description_t = {}
-
-      if ip_score ~= 0 then
-        total_score = total_score + ip_score
-        table.insert(description_t, 'ip: ' .. '(' .. math.floor(ip_score * 1000) / 100 .. ')')
-      end
-      if ipnet_score ~= 0 then
-        total_score = total_score + ipnet_score
-        table.insert(description_t, 'ipnet: ' .. ipnet .. '(' .. math.floor(ipnet_score * 1000) / 100 .. ')')
-      end
-      if asn_score ~= 0 then
-        total_score = total_score + asn_score
-        table.insert(description_t, 'asn: ' .. asn .. '(' .. math.floor(asn_score * 1000) / 100 .. ')')
-      end
-      if country_score ~= 0 then
-        total_score = total_score + country_score
-        table.insert(description_t, 'country: ' .. country .. '(' .. math.floor(country_score * 1000) / 100 .. ')')
-      end
-
-      if options['max_score'] and (total_score*10) > options['max_score'] then total_score = options['max_score']/10 end
-      if options['min_score'] and (total_score*10) < options['min_score'] then total_score = options['min_score']/10 end
-
-      if total_score ~= 0 then
-        task:insert_result(options['symbol'], total_score, table.concat(description_t, ', '))
-      end
     end
   end
 
-  local function create_get_command(ip, asn, country, ipnet)
+  local function create_get_command()
     local cmd = 'HMGET'
 
     local args = {options['hash']}
@@ -298,7 +293,10 @@ local ip_score_check = function(task)
     return cmd, args
   end
 
-  local ip = task:get_from_ip()
+  if task:get_user() or (ip and ip:is_local()) then
+    rspamd_logger.infox(task, "skip IP Score for local networks and authorized users")
+    return
+  end
   if ip:is_valid() then
     -- Check IP whitelist
     if whitelist then
@@ -317,44 +315,63 @@ local ip_score_check = function(task)
       end
     end
 
-    local cmd, args = create_get_command(ip, asn, country, ipnet)
-    local upstream = upstreams:get_upstream_by_hash(
-      ip_score_hash_key(asn, country, ipnet, ip))
-    local addr = upstream:get_addr()
-    rspamd_redis.make_request(task, addr, ip_score_redis_cb, cmd, args)
+    local cmd, args = create_get_command()
+
+    local ret = rspamd_redis_make_request(task,
+      redis_params, -- connect params
+      ip_score_hash_key(asn, country, ipnet, ip), -- hash key
+      false, -- is write
+      ip_score_redis_cb, --callback
+      cmd, -- command
+      args -- arguments
+    )
+    if not ret then
+      rspamd_logger.errx(task, 'error connecting to redis')
+    end
   end
 end
 
 
 -- Configuration options
 local configure_ip_score_module = function()
-  local opts =  rspamd_config:get_all_opt('ip_score')
-  if opts then
-    for k,v in pairs(opts) do
-      options[k] = v
+  local opts = rspamd_config:get_all_opt('options')
+  if opts and type(opts) ~= 'table' then
+    if type(opts['check_authed']) == 'boolean' then
+      check_authed = opts['check_authed']
     end
-    if options['servers'] and options['servers'] ~= '' then
-      upstreams = upstream_list.create(rspamd_config, options['servers'], default_port)
-      if not upstreams then
-        rspamd_logger.infox(rspamd_config, 'no servers are specified')
-      end
-    end
-    if options['whitelist'] then
-      whitelist = rspamd_config:add_radix_map(opts['whitelist'])
-    end
-    if options['asn_cc_whitelist'] then
-      asn_cc_whitelist = rspamd_config:add_hash_map(opts['asn_cc_whitelist'])
+    if type(opts['check_local']) == 'boolean' then
+      check_local = opts['check_local']
     end
   end
+  opts = rspamd_config:get_all_opt('ip_score')
+  if not opts then return end
+  for k,v in pairs(opts) do
+    options[k] = v
+  end
+  redis_params = rspamd_parse_redis_server('ip_score')
+  if not redis_params then
+    rspamd_logger.infox(rspamd_config, 'no servers are specified')
+    return
+  end
+  whitelist = rspamd_map_add('ip_score', 'asn_cc_whitelist', 'map',
+    'IP score whitelisted ips')
+  asn_cc_whitelist = rspamd_map_add('ip_score', 'whitelist', 'radix',
+    'IP score whitelisted ASNs/countries')
+  return true
 end
 
 
-configure_ip_score_module()
-if upstreams then
+if not configure_ip_score_module() then return end
+if redis_params then
   -- Register ip_score module
-  if options['asn_provider'] then
-    rspamd_config:register_pre_filter(asn_check)
-  end
-  rspamd_config:register_symbol(options['symbol'], 1.0, ip_score_check)
-  rspamd_config:register_post_filter(ip_score_set)
+  rspamd_config:register_symbol({
+    name = 'IPSCORE_SAVE',
+    type = 'postfilter',
+    priority = 5,
+    callback = ip_score_set,
+  })
+  rspamd_config:register_symbol({
+    name = options['symbol'],
+    callback = ip_score_check,
+  })
 end
