@@ -17,7 +17,6 @@ limitations under the License.
 -- Module for checking mail list headers
 
 local symbol = 'MAILLIST'
-local rspamd_logger = require "rspamd_logger"
 -- EZMLM
 -- Mailing-List: .*run by ezmlm
 -- Precedence: bulk
@@ -168,7 +167,7 @@ end
 
 -- RFC 2369 headers
 local function check_rfc2369(task)
-  header = task:get_header('List-Unsubscribe')
+  local header = task:get_header('List-Unsubscribe')
   if not header or not string.find(header, '<.+>') then
     return false
   end
@@ -219,7 +218,7 @@ local function check_ml_majordomo(task)
     return false
   end
 
-  local header = task:get_header('Precedence')
+  header = task:get_header('Precedence')
   if not header or (header ~= 'list' and header ~= 'bulk') then
     return false
   end
@@ -267,16 +266,16 @@ local function check_maillist(task)
     task:insert_result(symbol, 0.5, 'generic')
   end
 end
--- Registration
-if type(rspamd_config.get_api_version) ~= 'nil' then
-  if rspamd_config:get_api_version() >= 1 then
-    rspamd_config:register_module_option('maillist', 'symbol', 'string')
-  end
-end
+
+
 -- Configuration
-local opts =  rspamd_config:get_all_opt('maillist')if opts then
+local opts =  rspamd_config:get_all_opt('maillist')
+if opts then
   if opts['symbol'] then
     symbol = opts['symbol']
-    rspamd_config:register_symbol(symbol, 1.0, check_maillist)
+    rspamd_config:register_symbol({
+      name = symbol,
+      callback = check_maillist
+    })
   end
 end
