@@ -24,11 +24,16 @@ local bad_hosts = {}
 local good_hosts = {}
 local whitelist = nil
 local rspamd_logger = require "rspamd_logger"
+local check_local = false
+local check_authed = false
 
 local function check_quantity_received (task)
   local recvh = task:get_received_headers()
 
-  local function recv_dns_cb(resolver, to_resolve, results, err)
+  local function recv_dns_cb(_, to_resolve, results, err)
+    if err and (err ~= 'requested record is not found' and err ~= 'no records with this name') then
+      rspamd_logger.errx(task, 'error looking up %s: %s', to_resolve, err)
+    end
     task:inc_dns_req()
 
     if not results then
@@ -59,33 +64,30 @@ local function check_quantity_received (task)
     end
   end
 
-  if task:get_user() ~= nil then
+  local task_ip = task:get_ip()
+
+  if ((not check_authed and task:get_user()) or
+      (not check_local and task_ip and task_ip:is_local())) then
+    rspamd_logger.infox(task, 'Skipping once_received for authenticated user or local network')
+    return
+  end
+  if whitelist and task_ip and whitelist:get_key(task_ip) then
+    rspamd_logger.infox(task, 'whitelisted mail from %s',
+      task_ip:to_string())
     return
   end
 
-  if whitelist then
-    local addr = task:get_from_ip()
-
-    if addr and whitelist:get_key(addr) then
-      rspamd_logger.infox(task, 'whitelisted mail from %s',
-        addr:to_string())
-      return
-    end
-  end
-
-  local task_ip = task:get_ip()
   local hn = task:get_hostname()
-
   -- Here we don't care about received
   if (not hn or hn == 'unknown') and task_ip and task_ip:is_valid() then
     task:get_resolver():resolve_ptr({task = task,
       name = task_ip:to_string(),
-      callback = recv_dns_cb
+      callback = recv_dns_cb,
+      forced = true
     })
     return
   end
 
-  local recvh = task:get_received_headers()
   if recvh and #recvh <= 1 then
     local ret = true
     local r = recvh[1]
@@ -94,13 +96,12 @@ local function check_quantity_received (task)
       return
     end
 
-    local hn = nil
     if r['real_hostname'] then
-      hn = string.lower(r['real_hostname'])
+      local rhn = string.lower(r['real_hostname'])
       -- Check for good hostname
-      if hn and good_hosts then
+      if rhn and good_hosts then
         for _,gh in ipairs(good_hosts) do
-          if string.find(hn, gh) then
+          if string.find(rhn, gh) then
             ret = false
             break
           end
@@ -138,13 +139,25 @@ if type(rspamd_config.get_api_version) ~= 'nil' then
   end
 end
 
+local opts = rspamd_config:get_all_opt('options')
+if opts and type(opts) ~= 'table' then
+  if type(opts['check_local']) == 'boolean' then
+    check_local = opts['check_local']
+  end
+  if type(opts['check_authed']) == 'boolean' then
+    check_authed = opts['check_authed']
+  end
+end
 -- Configuration
-local opts =  rspamd_config:get_all_opt('once_received')
+opts = rspamd_config:get_all_opt('once_received')
 if opts then
   if opts['symbol'] then
-    local symbol = opts['symbol']
+    symbol = opts['symbol']
 
-    local id = rspamd_config:register_symbol(symbol, 1.0, check_quantity_received)
+    local id = rspamd_config:register_symbol({
+      name = symbol,
+      callback = check_quantity_received,
+    })
 
     for n,v in pairs(opts) do
       if n == 'symbol_strict' then
@@ -164,10 +177,20 @@ if opts then
           good_hosts = v
         end
       elseif n == 'whitelist' then
-        whitelist = rspamd_config:add_radix_map (v, 'once received whitelist')
+        whitelist = rspamd_map_add('once_received', 'whitelist', 'radix',
+          'once received whitelist')
       end
     end
-    rspamd_config:register_virtual_symbol(symbol_rdns, 1.0, id)
-    rspamd_config:register_virtual_symbol(symbol_strict, 1.0, id)
+
+    rspamd_config:register_symbol({
+      name = symbol_rdns,
+      type = 'virtual',
+      parent = id
+    })
+      rspamd_config:register_symbol({
+      name = symbol_strict,
+      type = 'virtual',
+      parent = id
+    })
   end
 end
