@@ -18,21 +18,17 @@ limitations under the License.
 -- Settings documentation can be found here:
 -- https://rspamd.com/doc/configuration/settings.html
 
-local set_section = rspamd_config:get_all_opt("settings")
+local rspamd_logger = require "rspamd_logger"
+local redis_params
 
-local settings = {
-  [1] = {},
-  [2] = {},
-  [3] = {}
-}
+local settings = {}
 local settings_ids = {}
 local settings_initialized = false
 local max_pri = 0
-local rspamd_logger = require "rspamd_logger"
 local rspamd_ip = require "rspamd_ip"
 local rspamd_regexp = require "rspamd_regexp"
 local ucl = require "ucl"
-require "fun" ()
+local fun = require "fun"
 
 -- Checks for overrided settings within query params and returns 'true' if
 -- settings are overrided
@@ -46,6 +42,8 @@ local function check_query_settings(task)
       task:set_settings(parser:get_object())
 
       return true
+    else
+      rspamd_logger.errx(task, 'Parse error: %s', err)
     end
   end
 
@@ -95,17 +93,17 @@ local function check_settings(task)
   local function check_addr_setting(rule, addr)
     local function check_specific_addr(elt)
       if rule['name'] then
-        if elt['addr'] == rule['name'] then
+        if rule['name']:lower() == elt['addr']:lower() then
           return true
         end
       end
       if rule['user'] then
-        if rule['user'] == elt['user'] then
+        if rule['user']:lower() == elt['user']:lower() then
           return true
         end
       end
-      if rule['domain'] then
-        if rule['domain'] == elt['domain'] then
+      if rule['domain'] and elt['domain'] then
+        if rule['domain']:lower() == elt['domain']:lower() then
           return true
         end
       end
@@ -139,9 +137,19 @@ local function check_settings(task)
     return false
   end
 
-  local function check_specific_setting(name, rule, ip, from, rcpt, user)
+  local function check_specific_setting(_, rule, ip, client_ip, from, rcpt,
+      user, auth_user)
     local res = false
 
+    if rule['authenticated'] then
+      if auth_user then
+        res = true
+      end
+      if not res then
+        return nil
+      end
+    end
+
     if rule['ip'] then
       if not ip then
         return nil
@@ -157,6 +165,21 @@ local function check_settings(task)
       end
     end
 
+    if rule['client_ip'] then
+      if not client_ip or not client_ip:is_valid() then
+        return nil
+      end
+      for _, i in ipairs(rule['client_ip']) do
+        res = check_ip_setting(i, client_ip)
+        if res then
+          break
+        end
+      end
+      if not res then
+        return nil
+      end
+    end
+
     if rule['from'] then
       if not from then
         return nil
@@ -202,6 +225,19 @@ local function check_settings(task)
       end
     end
 
+    if rule['request_header'] then
+      for k, v in pairs(rule['request_header']) do
+        local h = task:get_request_header(k)
+        res = (h and v:match(h))
+        if res then
+          break
+        end
+      end
+      if not res then
+        return nil
+      end
+    end
+
     if res then
       if rule['whitelist'] then
         rule['apply'] = {whitelist = true}
@@ -225,37 +261,40 @@ local function check_settings(task)
 
   rspamd_logger.infox(task, "check for settings")
   local ip = task:get_from_ip()
+  local client_ip = task:get_client_ip()
   local from = task:get_from()
   local rcpt = task:get_recipients()
   local uname = task:get_user()
   local user = {}
   if uname then
     user[1] = {}
-    for localpart, domainpart in string.gmatch(uname, "(.+)@(.+)") do
+    local localpart, domainpart = string.gmatch(uname, "(.+)@(.+)")()
+    if localpart then
       user[1]["user"] = localpart
       user[1]["domain"] = domainpart
       user[1]["addr"] = uname
-      break
-    end
-    if not user[1]["addr"] then
+    else
       user[1]["user"] = uname
       user[1]["addr"] = uname
     end
   end
   -- Match rules according their order
+  local applied = false
+
   for pri = max_pri,1,-1 do
-    if settings[pri] then
-      for name, r in pairs(settings[pri]) do
-        local rule = check_specific_setting(name, r, ip, from, rcpt, user)
+    if not applied and settings[pri] then
+      for _,s in ipairs(settings[pri]) do
+        local rule = check_specific_setting(s.name, s.rule, ip, client_ip, from, rcpt, user, uname)
         if rule then
           rspamd_logger.infox(task, "<%1> apply settings according to rule %2",
-            task:get_message_id(), name)
+            task:get_message_id(), s.name)
           if rule['apply'] then
             task:set_settings(rule['apply'])
+            applied = true
           end
           if rule['symbols'] then
             -- Add symbols, specified in the settings
-            each(function(val)
+            fun.each(function(val)
               task:insert_result(val, 1.0)
             end, rule['symbols'])
           end
@@ -298,7 +337,7 @@ local function process_settings_table(tbl)
       local out = {}
 
       if type(ip) == "table" then
-        for i,v in ipairs(ip) do
+        for _,v in ipairs(ip) do
           table.insert(out, process_ip(v))
         end
       elseif type(ip) == "string" then
@@ -337,7 +376,7 @@ local function process_settings_table(tbl)
     local function process_addr(addr)
       local out = {}
       if type(addr) == "table" then
-        for i,v in ipairs(addr) do
+        for _,v in ipairs(addr) do
           table.insert(out, process_addr(v))
         end
       elseif type(addr) == "string" then
@@ -373,8 +412,8 @@ local function process_settings_table(tbl)
       return out
     end
 
-    local check_table = function(elt, out)
-      if type(elt) == 'string' then
+    local check_table = function(chk_elt, out)
+      if type(chk_elt) == 'string' then
         return {out}
       end
 
@@ -390,6 +429,13 @@ local function process_settings_table(tbl)
         out['ip'] = check_table(elt['ip'], ip)
       end
     end
+    if elt['client_ip'] then
+      local ip = process_ip(elt['client_ip'])
+
+      if ip then
+        out['client_ip'] = check_table(elt['client_ip'], ip)
+      end
+    end
     if elt['from'] then
       local from = process_addr(elt['from'])
 
@@ -409,6 +455,22 @@ local function process_settings_table(tbl)
         out['user'] = check_table(elt['user'], user)
       end
     end
+    if elt['authenticated'] then
+      out['authenticated'] = true
+    end
+    if elt['request_header'] then
+      local rho = {}
+      for k, v in pairs(elt['request_header']) do
+        local re = rspamd_regexp.get_cached(v)
+        if not re then
+          re = rspamd_regexp.create_cached(v)
+        end
+        if re then
+          rho[k] = re
+        end
+      end
+      out['request_header'] = rho
+    end
 
     -- Now we must process actions
     if elt['symbols'] then out['symbols'] = elt['symbols'] end
@@ -432,7 +494,7 @@ local function process_settings_table(tbl)
 
   settings_initialized = false
   -- filter trash in the input
-  local ft = filter(
+  local ft = fun.filter(
     function(_, elt)
       if type(elt) == "table" then
         return true
@@ -444,9 +506,9 @@ local function process_settings_table(tbl)
   max_pri = 0
   local nrules = 0
   settings_ids = {}
-  for k,v in pairs(settings) do settings[k]={} end
+  for k in pairs(settings) do settings[k]={} end
   -- fill new settings by priority
-  for_each(function(k, v)
+  fun.for_each(function(k, v)
     local pri = get_priority(v)
     if pri > max_pri then max_pri = pri end
     if not settings[pri] then
@@ -454,10 +516,14 @@ local function process_settings_table(tbl)
     end
     local s = process_setting_elt(k, v)
     if s then
-      settings[pri][k] = s
+      table.insert(settings[pri], {name = k, rule = s})
       nrules = nrules + 1
     end
   end, ft)
+  -- sort settings with equal priopities in alphabetical order
+  for pri,_ in pairs(settings) do
+    table.sort(settings[pri], function(a,b) return a.name < b.name end)
+  end
 
   settings_initialized = true
   rspamd_logger.infox(rspamd_config, 'loaded %1 elements of settings', nrules)
@@ -467,7 +533,6 @@ end
 
 -- Parse settings map from the ucl line
 local function process_settings_map(string)
-  local ucl = require "ucl"
   local parser = ucl.parser()
   local res,err = parser:parse_string(string)
   if not res then
@@ -484,6 +549,83 @@ local function process_settings_map(string)
   return res
 end
 
+local function gen_redis_callback(handler, id)
+  return function(task)
+    local key = handler(task)
+
+    local function redis_settings_cb(err, data)
+      if not err and type(data) == 'string' then
+        local parser = ucl.parser()
+        local res,ucl_err = parser:parse_string(data)
+        if not res then
+          rspamd_logger.warnx(rspamd_config, 'cannot parse settings from redis: %s',
+            ucl_err)
+        else
+          local obj = parser:get_object()
+          rspamd_logger.infox(task, "<%1> apply settings according to redis rule %2",
+            task:get_message_id(), id)
+          task:set_settings(obj)
+        end
+      end
+    end
+
+    if not key then
+      rspamd_logger.errx(rspamd_config, 'Cannot execute handler number %s', id)
+      return
+    end
+
+    local ret,_,_ = rspamd_redis_make_request(task,
+      redis_params, -- connect params
+      key, -- hash key
+      false, -- is write
+      redis_settings_cb, --callback
+      'GET', -- command
+      {key} -- arguments
+    )
+    if not ret then
+      rspamd_logger.errx(task, 'Redis GET failed: %s', ret)
+    end
+  end
+end
+
+local redis_section = rspamd_config:get_all_opt("settings_redis")
+local redis_key_handlers = {}
+
+if redis_section then
+  redis_params = rspamd_parse_redis_server('settings_redis')
+  if redis_params then
+    local handlers = redis_section.handlers
+
+    for id,h in pairs(handlers) do
+      local chunk,err = load(h)
+
+      if not chunk then
+        rspamd_logger.errx(rspamd_config, 'Cannot load handler from string: %s',
+            tostring(err))
+      else
+        local res,func = pcall(chunk)
+        if not res then
+          rspamd_logger.errx(rspamd_config, 'Cannot add handler from string: %s',
+            tostring(func))
+        else
+          redis_key_handlers[id] = func
+        end
+      end
+    end
+  end
+
+  fun.each(function(id, h)
+    rspamd_config:register_symbol({
+      name = 'REDIS_SETTINGS' .. tostring(id),
+      type = 'prefilter',
+      callback = gen_redis_callback(h, id),
+      priority = 10
+    })
+  end, redis_key_handlers)
+end
+
+local set_section = rspamd_config:get_all_opt("settings")
+
 if set_section and set_section[1] and type(set_section[1]) == "string" then
   -- Just a map of ucl
   if not rspamd_config:add_map(set_section[1], "settings map", process_settings_map) then
@@ -493,4 +635,9 @@ elseif set_section and type(set_section) == "table" then
   process_settings_table(set_section)
 end
 
-rspamd_config:register_pre_filter(check_settings)
+rspamd_config:register_symbol({
+  name = 'SETTINGS_CHECK',
+  type = 'prefilter',
+  callback = check_settings,
+  priority = 10
+})
