@@ -16,8 +16,7 @@ limitations under the License.
 
 local rspamd_logger = require "rspamd_logger"
 local rspamd_util = require "rspamd_util"
-local ucl = require "ucl"
-require "fun" ()
+local fun = require "fun"
 
 local options = {
   dmarc_allow_symbol = 'DMARC_POLICY_ALLOW',
@@ -27,6 +26,8 @@ local options = {
   rules = {}
 }
 
+local E = {}
+
 local function whitelist_cb(symbol, rule, task)
 
   local domains = {}
@@ -44,6 +45,19 @@ local function whitelist_cb(symbol, rule, task)
         table.insert(domains, dom)
         return true,mult
       end
+    elseif rule['maps'] then
+      for map, mmult in pairs(rule['maps']) do
+        local val = map:get_key(dom)
+        if val then
+          if #val > 0 then
+            mult = tonumber(val)
+          else
+            mult = mmult
+          end
+          table.insert(domains, dom)
+          return true,mult
+        end
+      end
     else
       mult = rule['domains'][dom]
       if mult then
@@ -55,20 +69,26 @@ local function whitelist_cb(symbol, rule, task)
     return false,0.0
   end
 
-  local from = task:get_from(1)
   local found = false
   local mult = 1.0
+  local spf_violated = false
+  local dkim_violated = false
+  local dmarc_violated = false
 
   if rule['valid_spf'] then
     if not task:has_symbol(options['spf_allow_symbol']) then
       -- Not whitelisted
-      return
+      if not rule['blacklist'] or rule['strict'] then
+        return
+      end
+
+      spf_violated = true
     end
 
     -- Now we can check from domain or helo
     local from = task:get_from(1)
 
-    if from and from[1] and from[1]['domain'] then
+    if ((from or E)[1] or E).domain then
       local tld = rspamd_util.get_tld(from[1]['domain'])
 
       if tld then
@@ -90,12 +110,16 @@ local function whitelist_cb(symbol, rule, task)
   if rule['valid_dkim'] then
     local sym = task:get_symbol(options['dkim_allow_symbol'])
     if not sym then
-      return
+      if not rule['blacklist'] or rule['strict'] then
+        return
+      end
+
+      dkim_violated = true
     end
 
     local dkim_opts = sym[1]['options']
     if dkim_opts then
-      each(function(val)
+      fun.each(function(val)
         if not found then
           local tld = rspamd_util.get_tld(val)
 
@@ -109,11 +133,15 @@ local function whitelist_cb(symbol, rule, task)
 
   if rule['valid_dmarc'] then
     if not task:has_symbol(options['dmarc_allow_symbol']) then
-      return
+      if not rule['blacklist'] or rule['strict'] then
+        return
+      end
+
+      dmarc_violated = true
     end
     local from = task:get_from(2)
 
-    if from and from[1] and from[1]['domain'] then
+    if ((from or E)[1] or E).domain then
       local tld = rspamd_util.get_tld(from[1]['domain'])
 
       if tld then
@@ -123,7 +151,28 @@ local function whitelist_cb(symbol, rule, task)
   end
 
   if found then
-    task:insert_result(symbol, mult, domains)
+    if not rule['blacklist'] or rule['strict'] then
+      task:insert_result(symbol, mult, domains)
+    else
+      -- Additional constraints for blacklist
+      if rule['valid_spf'] or rule['valid_dkim'] or rule['valid_dmarc'] then
+        if dmarc_violated or dkim_violated or spf_violated then
+
+          if rule['strict'] then
+            -- Inverse multiplier to convert whitelist to blacklist
+            mult = -mult
+          end
+
+          task:insert_result(symbol, mult, domains)
+        elseif rule['strict'] then
+          -- Add whitelist score (negative)
+          task:insert_result(symbol, mult, domains)
+        end
+      else
+        -- Unconstrained input
+        task:insert_result(symbol, mult, domains)
+      end
+    end
   end
 
 end
@@ -134,28 +183,19 @@ local function gen_whitelist_cb(symbol, rule)
   end
 end
 
-local function process_whitelist_map(input)
-  local parser = ucl.parser()
-  local res,err = parser:parse_string(string)
-  if not res then
-    rspamd_logger.warnx(rspamd_config, 'cannot parse settings map: ' .. err)
-  else
-    local obj = parser:get_object()
-
-    options['rules'] = obj
-  end
-end
-
 local configure_whitelist_module = function()
   local opts =  rspamd_config:get_all_opt('whitelist')
   if opts then
     for k,v in pairs(opts) do
       options[k] = v
     end
+  else
+    rspamd_logger.infox(rspamd_config, 'Module is unconfigured')
+    return
   end
 
   if options['rules'] then
-    each(function(symbol, rule)
+    fun.each(function(symbol, rule)
       if rule['domains'] then
         if type(rule['domains']) == 'string' then
           rule['map'] = rspamd_config:add_kv_map(rule['domains'],
@@ -163,16 +203,26 @@ local configure_whitelist_module = function()
         elseif type(rule['domains']) == 'table' then
           -- Transform ['domain1', 'domain2' ...] to indexes:
           -- {'domain1' = 1, 'domain2' = 1 ...]
-          rule['domains'] = tomap(map(function(d)
-            local name = d
+          rule['maps'] = fun.tomap(fun.map(function(d)
+            local name
             local value = 1
 
             if type(d) == 'table' then
-              name = d[1]
+              name = rspamd_config:add_map({
+                url = d[1],
+                type = 'map',
+              })
               value = tonumber(d[2])
+            else
+              name = rspamd_config:add_map({
+                url = d,
+                type = 'map',
+              })
             end
 
-            return name,value
+            if name and value then
+              return name,value
+            end
           end, rule['domains']))
         else
           rspamd_logger.errx(rspamd_config, 'whitelist %s has bad "domains" value',
@@ -180,8 +230,16 @@ local configure_whitelist_module = function()
           return
         end
 
-        local id = rspamd_config:register_symbol(symbol, -1.0,
-          gen_whitelist_cb(symbol, rule))
+        local flags = 'nice,empty'
+        if rule['blacklist'] then
+          flags = 'empty'
+        end
+
+        local id = rspamd_config:register_symbol({
+          name = symbol,
+          flags = flags,
+          callback = gen_whitelist_cb(symbol, rule)
+        })
 
         if rule['valid_spf'] then
           rspamd_config:register_dependency(id, options['spf_allow_symbol'])
